function [ rho_molar ] = Density_scCO2( T, P )
% Calculation of the molar density of supercritical CO2 depending on pressure and temperature
% equation:  modified Peng-Robinson EOS (Model II) of Hekayati et al. (2016)
% input: temperature [K], pressure [Pa]
% output: molar density [mol/m3]
% range of validity: P > 73.76 bar und T > 304.12 K
% AARD: 2,29%, MAPE: 10,75% (85 data points)

R = 8.3144621; % J/(mol K)  

% CO2 properties:
Tc =  304.12; % K   
Pc = 7.376e6 ; % Pa  

% reduced values
Pr = P/Pc;
Tr = T/Tc;

% fitted parameters
B1 = 1.0010;
B2 = 0.0524;
B3 = 0.2315;
B4 = 0.1286;
B5 = 0.3258;
B6 = 0.0704;
B7 = 0.9461;

alpha = B1 + log(Pr) * (B2 + B3* log(Tr) -B4 * (log(Tr))^2 - B5 * sin(B6 * (log(Pr))^2)) - B7 * sin(log(Tr));

a = 0.45724 * R^2 * Tc^2 * alpha / Pc;
b = 0.07780 * R * Tc / Pc;

A =  P *R^-2 * a.* (T.^(-2));
B = b * P * R^(-1) * T.^(-1);

poly = [1 -(1-B) (A-3*B^2 -2*B) -(A*B-B^2-B^3)];
Z_all = roots(poly);

for j=1:size(Z_all,1)  
    if isreal(Z_all(j))
        Z = Z_all(j);
    end   
end
    
v = Z*R*T/P; % m3/mol
rho_molar = 1/v; % mol/m3

end

