function [ D12 ] = Diffusion_coefficient_CO2_in_EtOH( T, P )
% Calculation of the diffusion coefficient of supercritical CO2 in ethanol depending on temperature and pressure
% equation:  correlation of Lito et al. 2013 
% input: temperature [K], pressure [Pa]
% output: binary diffusion coefficient [m2/s] at infinite dilution
% 1: scCO2     2: EtOH  % attention: in paper of Lito et al. 2013 1=solvent here EtOH,  2=solute here CO2
 
k_B = 1.380658*10^-16; % Boltzmann constant in g*cm^2/s^2*K
Avogadro = 6.0221367*10^23; % in mol^-1
R_g = 8.314459848; % molar gas constant in J/mol/K

% component properties
M1 = 44.01;  % g/mol
sigma1_LJ = 3.26192*10^-8; % cm
 
M2 = 46.069; % g/mol
sigma2_LJ = 4.26113*10^-8; % cm

E_D = 379.865; % J/mol
Epsi1_divided_by_k_B = 500.71;
Epsi2_divided_by_k_B = 1291.41;
Epsi12_divided_by_k_B = (Epsi1_divided_by_k_B*Epsi2_divided_by_k_B)^(1/2);
   
sigma12_LJ = (sigma1_LJ + sigma2_LJ)/2;
m12 =  1/Avogadro*M1*M2/(M1 + M2); %in g 

rho2 = 46.069*Density_EtOH_high_pressure( T, P )*10^-6; % g/cm^3
ndrho2 = Avogadro*Density_EtOH_high_pressure( T, P )*10^-6; % cm^-3

T1_star = T/ Epsi1_divided_by_k_B;
T2_star = T/ Epsi2_divided_by_k_B;
T12_star = T/ Epsi12_divided_by_k_B;

sigma1_eff = sigma1_LJ *2^(1/6)*(1+(1.3229*T1_star)^(1/2))^(-1/6); 
sigma2_eff = sigma2_LJ *2^(1/6)*(1+(1.3229*T2_star)^(1/2))^(-1/6);
sigma12_eff = sigma12_LJ *2^(1/6)*(1+(1.3229*T12_star)^(1/2))^(-1/6);

rho2_star_eff = ndrho2*sigma2_eff^3; 
phi2_eff = pi/6*rho2_star_eff;
 
a = -1.676382*rho2_star_eff + 1.638561; 
b = -8.516830*rho2_star_eff + 8.631536; 
c = -1.320347*rho2_star_eff + 1.351067; 
d = -5.062546*rho2_star_eff + 5.409662; 
F11 = 1 + 0.94605*rho2_star_eff^1.5 + 1.4022*rho2_star_eff^3 - 5.6898*rho2_star_eff^5 + 2.6626*rho2_star_eff^7; 
F12 = (F11 + rho2_star_eff^1.7*(a*log(sigma1_eff/sigma2_eff) + b*(log(sigma1_eff/sigma2_eff))^2 + c*log(M1/M2)))/(1 + rho2_star_eff^3*(d*log(sigma1_eff/sigma2_eff))^2);

g_sigmma12_eff = 1/(1-phi2_eff)^3*(1-phi2_eff+2*phi2_eff/(1+sigma2_eff/sigma1_eff))*(1-phi2_eff+phi2_eff/(1+sigma2_eff/sigma1_eff));

D12_cm2s = 3/(8*ndrho2*sigma12_eff^2)*(k_B*T/(2*pi*m12))^(1/2)*F12/g_sigmma12_eff*exp(-E_D/(R_g*T)); % in cm^2/s
D12 = D12_cm2s*10^-4; % in m^2/s
end

