function [ rho_molar ] = Mixture_density_CO2_EtOH( T, P, x1 )
% Calculation of the mixture molar density of CO2 and ETOH for single phase (supercritical CO2 and liquid ethanol)
% depending on pressure, temperature and composition
% equation: modified Peng-Robinson EOS (Model II) for CO2, VTPR EOS (c = 4.275*10^-6 m3/mol) for EtOH 
% 1 Parameter mixing rule (asymmetric) with concentration dependent interaction parameters
% input: temperature [K], pressure [Pa], molar fraction of CO2 [mol/mol]
% output: molar density [mol/m3]
% range of validity: above the critical point of the mixture
% AARD: 1,91%, MAPE: 12,34% (185 data points)

x = [x1; 1-x1];

R = 8.3144621; % J/(mol K)  
% CO2 properties:
Tc1 =  304.12; % K   
Pc1 = 7.376e6 ; % Pa  
% reduced values
Pr1 = P/Pc1;
Tr1 = T/Tc1;
% fitted parameters
B1 = 1.0010;
B2 = 0.0524;
B3 = 0.2315;
B4 = 0.1286;
B5 = 0.3258;
B6 = 0.0704;
B7 = 0.9461;

alpha1 = B1 + log(Pr1) * (B2 + B3* log(Tr1) -B4 * (log(Tr1))^2 - B5 * sin(B6 * (log(Pr1))^2)) - B7 * sin(log(Tr1));
a(1) = 0.45724 * R^2 * Tc1^2 * alpha1 / Pc1;
b(1) = 0.07780 * R * Tc1 / Pc1;
c(1) = 0;

% Ethanol properties:
Tc2 = 513.92; % K
Pc2 = 6.148e6; % Pa
omega2 = 0.649; 

kappa2 = 0.37464 + 1.54226*omega2 - 0.26992*omega2^2;
alpha2 = (1 + kappa2 * ( 1 - (T/Tc2).^0.5 )).^2;
a(2) = 0.45724 * R^2 * Tc2^2 * alpha2 / Pc2;
b(2) = 0.07780 * R * Tc2 / Pc2;
c(2) = -4.275e-6; %m3/mol

% mixing properties
k(1,1) = 0;
k(2,2) = 0;
k(1,2) = 0.156;
k(2,1) = 0.758;

a_mix = 0;
b_mix = 0;
c_mix = 0;
    for i=1:2  
        c_mix = c_mix + x(i) * c(i);
        b_mix = b_mix + x(i) * b(i);
        for j=1:2
            a_ij = sqrt(a(i) * a(j)) * (1 - x(i)*k(i,j) - x(j)*k(j,i));
            a_mix = a_mix + x(i) * x(j) * a_ij;
        end
    end
A_mix = a_mix * P / (R * T)^2; 
B_mix = b_mix * P / (R * T);

cubic_equ_a3 = 1;   
cubic_equ_a2 = B_mix - 1;
cubic_equ_a1 = A_mix - 3*B_mix^2 - 2*B_mix;
cubic_equ_a0 = -A_mix*B_mix + B_mix^2 + B_mix^3;
    
Z_all = roots([cubic_equ_a3 cubic_equ_a2 cubic_equ_a1 cubic_equ_a0]);  
for j=1:size(Z_all,1) 
    if isreal(Z_all(j))
        Z = Z_all(j);
    end
end

v = Z*R*T/P + c_mix; % m3/mol
rho_molar = 1/v; % mol/m3
end
