%% PackedBedFDMVersionB v 1.0.0, Ilka Selmer, Anna-Sophia Behnecke, Patricio Farrell, 2020
% This work is licensed under a Creative Commons 
% Attribution-NonCommercial-ShareAlike 4.0 International License.

% When using this software, cite it with
%
%   Selmer, Ilka; Behnecke, Anna-Sophia; Farrell, Patricio (2020): 
%   Mass transport model describing the supercritical drying kinetics of gel 
%   particles in a packed bed v.1.0.0. figshare. Software. 
%   https://doi.org/10.6084/m9.figshare.12504863
%
% MIT License
%
% Copyright (C) 2020, Ilka Selmer, Anna-Sophia Behnecke, Patricio Farrell, 
% <ilka.selmer@tuhh.de>
%
% Permission is hereby granted, free of charge, to any person obtaining a copy
% of this software and associated documentation files (the "Software"), to deal
% in the Software without restriction, including without limitation the rights
% to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
% copies of the Software, and to permit persons to whom the Software is
% furnished to do so, subject to the following conditions:
%
% The above copyright notice and this permission notice shall be included in all
% copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
% IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
% AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
% OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
% SOFTWARE.
%
% The code calculates the supercritical drying time of gel particles in a 
% packed bed using a finite difference method.  

% Input parameters are marked.

close all;
clearvars;
clc;

% Process conditions
P = 10e+6; % INPUT pressure in Pa
T = 321; % INPUT temperature in K
m_dot = 0.2003; % INPUT outlet mass flow in g/s
m_dot = m_dot/1000; % kg/s

% Initial and end conditions in the gel
x2_start = 1; % INPUT initial ethanol molar fraction within the gel body
x1_start = 1-x2_start; 
x2_end = y_EtOH_max_G_phase( T ); % end condition in the gel, defines the drying end 

% Initial and end conditions in the bulk fluid
x2_F_start = 0.95; % INPUT initial ethanol molar fraction in the bulk fluid, should not be 0
x1_F_start = 1-x2_F_start;

% Gel properties
epsilon = 0.93; % INPUT gel porosity
tau = 3.48; % INPUT gel tortuosity
R = 3.175e-3; % INPUT radius of gel sphere in m
D21 = Diffusion_coefficient_EtOH_in_CO2(T,P)*epsilon/tau; % effective diffusion coefficient of ethanol in CO2 at infinite dilution m^2/s
D12 = Diffusion_coefficient_CO2_in_EtOH(T,P)*epsilon/tau; % effective diffusion coefficient of CO2 in ethanol at infinite dilution m^2/s

% Discretization of the gel sphere
f = 25; % INPUT number of volume elements (the higher f, the lower must be dt and the higher is the calculation effort)
dr = 1/f*R; % step in radial direction
dt = 0.05; % INPUT time step (stabilty condition is depending on dr and beta; if the value is chosen too high, the program stops with an error during calculating of a new x)
N = f+1; % number of grid nodes of the gel body in radial direction

% Properties of the packed bed
phi = 0.4; % INPUT packed bed porosity
V_total = 1.514e-4; %INPUT total volume of the packed bed in m^3
V_fluid = phi*V_total; % volume of the bulk fluid in m^3
V_particle = 4/3*pi*R^3; % volume of one gel particle in m^3
N_particle = (1-phi)*V_total/V_particle; % total number of particles in the packed bed
g = 9.80665; % m/s^2
D_ac = 2.1e-2; % INPUT diameter of the cylindrical autoclav in m
A_ac = pi*D_ac^2/4; % cross-sectional area in m^2
tau_F = 1; % INPUT tortuosity of the packed bed 

% Discretization of the packed bed in axial direction 
S = 20; % INPUT number of grid nodes of the packed bed body in axial direction
dx = V_total/A_ac/(S-1);

% pre-sizing of the solution matrices (to reduce the calculation time) 
K = 40000; % INPUT guessed matrix size (K>k), guessed K>guessed drying time/dt 

%%% Parameters within the gel
C2 = zeros(N,K,S);
x2 = zeros(N,K,S);
x1 = zeros(N,K,S);
D = zeros(N,K,S);
C = zeros(N,K,S);

%%% Parameters in the packed bed or bulk fluid
C2_Fs_1 = zeros (1,K);
x2_Fs_1 = zeros (1,K);
x1_Fs_1 = zeros (1,K);
C_Fs_1 = zeros (1,K);
rho_Fs_1 = zeros (1,K);
V_dots_1 = zeros (1,K);
v_beds_1 = zeros (1,K);
cn = zeros(1,K);
D_L_mean = zeros (1,K);
C2_F = zeros(S,K);
C_F = zeros(S,K);
x2_F = zeros(S,K);
x1_F = zeros(S,K);
rho_F = zeros (S,K);
D_F = zeros (S,K);
V_dot = zeros (S,K); 
v = zeros (S,K);
source = zeros(S,K);
v_bed = zeros (S,K);
Re = zeros (S,K);
Sc = zeros (S,K);
Gr = zeros (S,K);
Sh = zeros (S,K);
beta = zeros (S,K);
eta = zeros (S,K);
nu = zeros (S,K);
D_L = zeros (S,K);

%%% Parameters at the gels surface
rho_N1 = zeros (S,K);
c2_N1 = zeros (S,K);
x2_N1 = zeros (S,K);
x1_N1 = zeros (S,K);
c_N1 = zeros (S,K);
D_N1 = zeros (S,K);
N2_P = zeros (S,K);
N2_out = zeros (S,K);

%%% Parameters for evaluation
N_total = zeros (1,K);
N1 = zeros (1,K);
N2_outlet_ac = zeros (1,K);

% Spherical volume elements
V = zeros(N,1);
for n = 2:N-1
    V(n) = 4/3 * pi* (((n-0.5)*dr)^3-((n-1.5)*dr)^3);
end
V(1) = 4/3* pi* (0.5*dr)^3;
V(N) = 4/3 * pi* (((N-1)*dr)^3-((N-1.5)*dr)^3);

% Initial condition within the gels
c_0 = Mixture_density_CO2_EtOH( T, P, x1_start);
c2_0 = c_0*x2_start;
C2(:,1,:) = c2_0; 
C(:,1,:) = c_0;
x2(:,1,:) = x2_start;

% Initial conditions of the packed bed or bulk fluid
x2_Fs_1(1) = x2_F_start;
x1_Fs_1(1) = 1-x2_Fs_1(1);
C_Fs_1(1) = Mixture_density_CO2_EtOH( T, P, x1_Fs_1(1)); 
rho_Fs_1(1) = C_Fs_1(1) .* (46.069 * x2_Fs_1(1) + 44.01 * (1-x2_Fs_1(1))) * 1e-3; % in kg/m^3 
V_dots_1(1) = m_dot/rho_Fs_1(1); % in m^3/s 
x2_F(:,1) = x2_F_start;
C_F_0 = Mixture_density_CO2_EtOH(T, P, x1_F_start);
C2_F_0 = C_F_0 * x2_F_start;
C_F(:,1) = C_F_0;
C2_F(:,1) = C2_F_0;
for s = 1:S
eta(s,1) = Mixture_viscosity_CO2_EtOH(T,P,1-x2_F(s,1));
D_F(s,1) = tau/epsilon*D12^(x2_F(s,1))*D21^(1-x2_F(s,1));
end
rho_F(:,1) = C_F(:,1).*(46.069*x2_F(:,1) + 44.01*(1-x2_F(:,1)))*1e-3; % in kg/m^3
nu(:,1) = eta(:,1)./rho_F(:,1);
V_dot(:,1) = m_dot./rho_F(:,1); % in m^3/s 
v(:,1) = V_dot(:,1)/A_ac; % in m/s
v_bed(:,1) = v(:,1)/phi;
v_beds_1(1) = V_dots_1(1)/A_ac/phi;
Re(:,1) = 2*R*v(:,1)./(phi*nu(:,1));
Sc(:,1) = nu(:,1)./D_F(:,1);
Gr(:,1) = 0; 
for s = 1:S
Sh(s,1) = Sheerwood_VDI_heat_atlas(Re(s,1), Sc(s,1), phi);
D_L(s,1) = Dispersion_coefficient( v(s,1), R, D_F(s,1), tau_F, Sc(s,1), Re(s,1) );
end
beta (:,1) = D_F(:,1).*Sh(:,1)/(2*R);
D_L_mean(1) = mean(D_L(1:S,1));

% Initial ethanol amount within one gel particle for each packed bed step s
for s = 1:S
N2_P(s,1) = sum(C2(:,1,s).*V*epsilon); 
end

% Matrix to calculate the diffusion within the gel particle
A = zeros(N,N);                
for m = 2:(N-1)
    for n = 1:N
        if n == m-1
            A(m,n) = 1-1/(m-1);
        end
        if m == n 
            A(m,n) = -2;
        end
        if n == m+1
            A(m,n) = 1+1/(m-1);
        end
    end
end
A(1,1) = -2;
A(1,2) = 2;

% Calculation of the mass transport

k = 1;


while   (max(max(x2(:,k,:)))> x2_end) % drying end condition fullfilled in all gel particles? 
 
    % calculation for each packed bed step
    for s = 1:S  
     %% calculation of the source term      
     % calculation of the effective diffusion coefficient within the gel using the new x2 
            for n = 1:N
                D(n,k,s) = D12^(x2(n,k,s)) * D21^(1-x2(n,k,s));
            end
              
        % boundary condition mass transfer
        % calculation of the ethanol concentration c2 at the boundary N+1
        c2_N1(s,k) = C2(N-1,k,s) - 2 * dr * beta(s,k)/D(N,k,s) * (C2(N,k,s) - C2_F(s,k)); 
        
        if (C2(N,k,s) - C2_F(s,k)) < 0
            c2_N1(s,k) = C2_F(s,k);
        end
        
        if c2_N1(s,k) < 0
            c2_N1(s,k) = 0;
        end
        x2_N1(s,k) = cal_x_EtOH_from_rho_EtOH_PR_MR2( T, P, c2_N1(s,k));
        x1_N1(s,k) = 1-x2_N1(s,k);
        c_N1(s,k) = Mixture_density_CO2_EtOH( T, P, x1_N1(s,k)); 
        D_N1(s,k)  = D12^(x2_N1(s,k)) * D21^(1-x2_N1(s,k));
        rho_N1(s,k) = c_N1(s,k) .* (46.069 * x2_N1(s,k) + 44.01 * (1-x2_N1(s,k))) * 1e-3; % in kg/m^3 
        
        % Vector  Dj-1 Dj+1
        Dj1 = zeros(N,1);
        Dj2 = zeros(N,1);
        for n = 2: (N-1)
            Dj1(n) = D((n-1),k,s);
            Dj2(n) =  D((n+1),k,s);
        end

       % Vector  cj-1 cj+1
        cj1 = zeros(N,1);
        cj2 = zeros(N,1);
        for n = 2: (N-1)
            cj1(n) = C((n-1),k,s);
            cj2(n) =  C((n+1),k,s);
        end
        
        % Vector  x2j-1 x2j+1
        x2j1 = zeros(N,1);
        x2j2 = zeros(N,1);
        for n = 2: (N-1)
            x2j1(n) = x2((n-1),k,s);
            x2j2(n) =  x2((n+1),k,s);
        end

        F = diag(C(:,k,s)) * (Dj1 - Dj2) + diag(D(:,k,s)) * (cj1 - cj2);
        
        C2(:,(k+1),s) = C2(:,k,s) + dt/dr^2 * ( diag( diag(D(:,k,s)) * C(:,k,s) )* ( A*x2(:,k,s) ) + 1/4 * ( diag(x2j1) * F - diag(x2j2) * F  ) ); 
        
        % using the boundary values
        C2(N,(k+1),s) = C2(N,k,s) + dt/dr^2 * ( ( diag(1-1/N)* x2(N-1,k,s) - 2* x2(N,k,s)+ (1 + 1/N) *x2_N1(s,k)) *(diag(D(N,k,s))*C(n,k,s)) + 1/4 * ( (diag(x2(N-1,k,s)-x2_N1(s,k))) *(diag(D(N-1,k,s)-D_N1(s,k)) *C(N,k,s) + diag((C(N-1,k,s) - c_N1(s,k))) * D(N,k,s)) ) );

        % calculation of ethanol molar fraction x2        
            for n = 1:N
                  x2(n,(k+1),s) = cal_x_EtOH_from_rho_EtOH_PR_MR2( T, P, C2(n,(k+1),s));
                  x1(n,(k+1),s) = 1-x2(n,(k+1),s); 
            end
            
        % calculation of mixture concentration C using x2
            for n = 1:N
                C(n,k+1,s) = Mixture_density_CO2_EtOH( T, P, x1(n,k+1,s)); 
            end
            
        % calculation of the source term (ethanol amount being transferred from the gel particles to the bulk fluid)
            N2_P(s,k+1) = sum(C2(:,k+1,s).*V*epsilon);
            N2_out(s,k+1) = N_particle/(S-1) * (N2_P(s,k) - N2_P(s,k+1)); 
            source(s,k+1) =  N2_out(s,k+1)/(dt*V_fluid/(S-1)); 
    end
            N2_out(1,k+1) = N_particle/(S-1) * 0.5 * (N2_P(1,k) - N2_P(1,k+1)); 
            source(1,k+1) =  N2_out(1,k+1)/(dt*V_fluid*0.5/(S-1));               
            N2_out(S,k+1) = N_particle/(S-1) * 0.5 * (N2_P(S,k) - N2_P(S,k+1)); 
            source(S,k+1) =  N2_out(S,k+1)/(dt*V_fluid*0.5/(S-1));   

        % boundary condition of the packed bed z=0 (s=-1)
            C2_Fs_1(k+1)= C2_F(1,k)*D_L_mean(k)/dx*1/(D_L_mean(k)/dx+m_dot/rho_Fs_1(k)/A_ac/phi);            
            x2_Fs_1(k+1) = cal_x_EtOH_from_rho_EtOH_PR_MR2( T, P, C2_Fs_1(k+1));
            x1_Fs_1(k+1) = 1-x2_Fs_1(k+1);
            C_Fs_1(k+1) = Mixture_density_CO2_EtOH( T, P, x1_Fs_1(k+1)); 
            rho_Fs_1(k+1) = C_Fs_1(k+1) .* (46.069 * x2_Fs_1(k+1) + 44.01 * (1-x2_Fs_1(k+1))) * 1e-3; % in kg/m^3 
            V_dots_1(k+1) = m_dot/rho_Fs_1(k+1); % in m^3/s 
            v_beds_1(k+1) = V_dots_1(k+1)/A_ac/phi;
            
        % calculation of the ethanol concentration in the bulk fluid using the FDM Version B
            [ A_FDM_VarB, b_FDM_VarB ] = FDM_function_VarB (v_bed(:,k), v_beds_1(k+1), D_L_mean(k), source(:,k+1), S, dx);
            C2_F(:,k+1) = C2_F(:,k) - dt./dx.*(A_FDM_VarB*C2_F(:,k) - b_FDM_VarB); 
            
        % Condition number
            if issparse(A_FDM_VarB)
               cn(k)= condest(A_FDM_VarB);
            else
               cn(k) = cond(A_FDM_VarB);
            end
            
    %% further calculations for the packed bed or bulk fluid
        % calculation for each packed bed step            
    for s = 1:S 
        x2_F(s,(k+1)) = cal_x_EtOH_from_rho_EtOH_PR_MR2( T, P, C2_F(s,k+1)); % ethanol molar fraction in the bulk fluid
        x1_F(s,k+1) = 1-x2_F(s,k+1); % CO2 molar fraction in the bulk fluid
        C_F(s,k+1) = Mixture_density_CO2_EtOH( T, P, x1_F(s,k+1)); % mixture concentration of the bulk fluid 
        eta(s,k+1) = Mixture_viscosity_CO2_EtOH(T,P,1-x2_F(s,k+1)); % mixture dynamic viscosity in Pa*s
        rho_F(s,k+1) = C_F(s,k+1) .* (46.069 * x2_F(s,k+1) + 44.01 * (1-x2_F(s,k+1))) * 1e-3; % density of the bulk fluid in kg/m^3 
        nu(s,k+1) = eta(s,k+1)./rho_F (s,k+1); % kinematic viscosity in m^2/s
        D_F (s,k+1) = tau/epsilon*D12^(x2_F(s,k+1)) * D21^(1-x2_F(s,k+1)); % diffusion coefficient in the bulk fluid in m^2/s
        V_dot(s,k+1) = m_dot/rho_F(s,k+1); % volume flow in m^3/s  
        v(s,k+1) = V_dot(s,k+1) / A_ac; % velocity though the empty autoclave in m/s
        v_bed(s,k+1) = v(s,k+1)/phi; % interstitial velocity in m/s
        Re(s,k+1) = 2*R*v(s,k+1) ./ (phi*nu(s,k+1)); % Reynolds number
        Sc(s,k+1) = nu(s,k+1) ./ D_F(s,k+1); % Schmidt number
        Gr(s,k+1) = (rho_N1 (s,k) - rho_F(s,k+1))* g * (2*R)^3 / (rho_F(s,k+1) * nu(s,k+1).^2); % Grashof number
        Gr(s,k+1) = abs (Gr(s,k+1));
        Sh(s,k+1) = Sheerwood_VDI_heat_atlas(Re(s,k+1), Sc(s,k+1), phi); % Sheerwood correlation 
        beta(s,k+1) = D_F (s,k+1).* Sh(s,k+1) / (2*R); % mass transfer coefficient in m/s 
        D_L(s,k+1) = Dispersion_coefficient(v(s,k+1), R, D_F(s,k+1), tau_F, Sc(s,k+1), Re(s,k+1)); % dispersion coefficient    
    end
    D_L_mean(k+1) = mean(D_L(1:S,k+1)); 
     
if mod(k, 1000) == 0
    disp(k);
    disp(max(max(x2(:,k,:))));
end

k = k+1;
end

t_end = (k-1)*dt/60 % calculated drying time in min

%% Mol balance
% Calculation of the amount of substances in one gel sphere depending on time
for t = 1:k-1
    N_total(t) = sum(C(:,t).*V*epsilon);
    N1(t) = N_total(t) - N2_P(t);
end

% Ethanol amount being extracted in total at the autoclave outlet depending on time
N2_outlet_ac(1) = V_dot(S,1)* C2_F(S,1) * dt/2;
for i=2:k-2
    N2_outlet_ac(i) = N2_outlet_ac(i-1) + V_dot(S,i).* C2_F(S,i) * dt;
end
N2_outlet_ac(k-1) = N2_outlet_ac(i) + V_dot(S,k-1).* C2_F(S,k-1) * dt/2;

% Initial ethanol amount within the gel particles 
N2_particle_start_s(1) = N2_P(1,1)*N_particle*0.5/(S-1);
for i = 2:S-1
   N2_particle_start_s(i) = N2_particle_start_s(i-1) + N2_P(i,1)*N_particle/(S-1) ;
end 
N2_particle_start_S = N2_P(S,1)*N_particle*0.5/(S-1);
N2_particle_start = N2_particle_start_s(S-1) + N2_particle_start_S;

% Ethanol amount within the gel particles at the drying end
N2_particle_end_s(1) = N2_P(1,k-1)*N_particle*0.5/(S-1);
for i = 2:S-1
   N2_particle_end_s(i) = N2_particle_end_s(i-1) + N2_P(i,k-1)*N_particle/(S-1);
end    
N2_particle_end_S = N2_P(S,k-1)*N_particle*0.5/(S-1);
N2_particle_end = N2_particle_end_s(S-1) + N2_particle_end_S;

% Initial ethanol amount in the bulk fluid 
N2_fluid_start_s(1) = C2_F(1,1)*V_fluid*0.5/(S-1);
for i = 2:S-1
    N2_fluid_start_s(i)= N2_fluid_start_s(i-1) + C2_F(i,1)*V_fluid/(S-1);
end 
N2_fluid_start_S = C2_F(S,1)*V_fluid*0.5/(S-1);
N2_fluid_start = N2_fluid_start_s(S-1) + N2_fluid_start_S;

% Ethanol amount in the bulk fluid at the drying end
N2_fluid_end_s(1) = C2_F(1,k-1)*V_fluid*0.5/(S-1);
for i = 2:S-1
    N2_fluid_end_s(i) = N2_fluid_end_s(i-1)+ C2_F(i,k-1)*V_fluid/(S-1);
end 
N2_fluid_end_S = C2_F(S,k-1)*V_fluid*0.5/(S-1);
N2_fluid_end = N2_fluid_end_s(S-1) + N2_fluid_end_S;

% Molar error
N2_end = N2_outlet_ac(k-1)+ N2_particle_end + N2_fluid_end;
N2_start = N2_particle_start + N2_fluid_start;
figure();
plot(N2_outlet_ac/N2_start);
Mol_error = (1-N2_end/N2_start)*100; %in Mol% of the initial total ethanol amount
figure();
hold on;
for i = 1:S
    plot(C2_F(i,:));
end

% save(''); % INPUT folder to save the calculation results 