% Solves the the 1D flux problem with the complete flux scheme. The matrix
% is the same as the Scharfetter-Gummel matrix but the right-hand side
% changes.
function [cCFS, ACFS, cn, b]=SolveWithCFSNeumann(D, v, s, domainLeft, domainRight, gridWholeDomain, gridSize, boxSize, numberGridPoints, boxes)

%% If v function handle, create a velocity vector
% If v is a scalar nothing changes

% if isa(v,'function_handle')
%   v = v(boxes);
% end

% TODO create exact solution for a given choice of v(x)!!!

%% Define numerical Peclt number
% Note: gridSize and P have N points

% cut off final component in v
vcut = v(1:end-1);
Pe = vcut./D.*gridSize;

%% Set up matrix
% This version is for nonsparse matrices, where each diagonal is
% implemented manually.
% diagonal =      diag( D./(gridSize(2:end)).*Bernoulli(-v./D*gridSize(2:end)) + D./(gridSize(1:end-1)).*Bernoulli(v/D*gridSize(1:end-1))); % c_i
% upperDiagonal = diag(-D./(gridSize(2:end-1)) .* Bernoulli( v./D*gridSize(2:end-1)), 1);  % c_{i+1}
% lowerDiagonal = diag(-D./(gridSize(2:end-1)) .* Bernoulli(-v./D*gridSize(2:end-1)),-1);  % c_{i-1}
% ASG = diagonal + lowerDiagonal + upperDiagonal;

%% Set up matrix
% This version sets up a sparse matrix according to: 
% http://de.mathworks.com/help/matlab/ref/spdiags.html
% Note that the lower and upper diagonal need to be padded accordingly!

% Set up three vectors for tridiagonal sparse matrix
diagVec      =  [D/gridSize(1)*Bernoulli(-Pe(1)), D./gridSize(2:end).*Bernoulli(-Pe(2:end)) + D./gridSize(1:end-1).*Bernoulli(Pe(1:end-1)), D./gridSize(end)*Bernoulli( Pe(end)) + v(end)];     % c_{j} changed in comparison to original version
upperDiagVec = [0,-D./gridSize .* Bernoulli( Pe)  ];                                        % c_{j+1}
lowerDiagVec = [  -D./gridSize .* Bernoulli(-Pe),0];                                        % c_{j-1}

% We overwrite the last (relevant) element according to the boundary
% condition
lowerDiagVec(end-1) = -D./gridSize(end)* Bernoulli(-Pe(end));                               % c_{j-1} changed in comparison to original version


ACFS = spdiags([lowerDiagVec' diagVec' upperDiagVec'],-1:1,numberGridPoints, numberGridPoints);

 
%% Set up source term vector that 
% It enters the right-hand side of the linear
% system due to nonlinear BVP and the inhomogeneous flux.

% In higher dimensions, this is too costly to do. Also, the Green's 
% function has been derived under the assumption that s is constant on each
% Voronoi box. So the extra precision here won't help us.
%
% Hence, the two approximations below.

% sIntegrated = zeros(1,numberInteriorGridPoints);
% 
% for i = 1:numberInteriorGridPoints
%     sIntegrated(i)=integral(s,boxes(i),boxes(i+1),'ArrayValued',1);
% end

% Note: sEvaluatedAtNotes is constructed on the whole grid (N+2 points)
sEvaluatedAtNodes = s; 
%sEvaluatedAtNodes = s(gridWholeDomain);                               

%% Three ways to define the vector S 

%%%%%
% 1) S = s evaluated at nodes
%    This is an O(h) approximation of sIntegrated, case (i)
%
 S_ev = sEvaluatedAtNodes;

%%%%%
% 2) S = integral mean of s on Voronoibox
%    This is another O(h) approximation of sIntegrated, case (ii) 
%    Auf uniformen Gittern ist dies fr s(x)=x nicht knotenexakt wie der 
%    erste Fall. Die Variante 3) stellt Knotenexaktheit wieder her.

% leftHalfBoxSize  = boxes(1) - domainLeft;
% rightHalfBoxSize = domainRight - boxes(end);
% integralLeftHalfBox = integral(s,domainLeft,boxes(1),'ArrayValued',1);
% integralRighttHalfBox = integral(s,boxes(end),domainRight,'ArrayValued',1);
% S = 1./[leftHalfBoxSize boxSize rightHalfBoxSize] .*[integralLeftHalfBox,  sIntegrated, integralRighttHalfBox];

%%%%%
% 3) S = integral mean of s on Voronoibox -- except for half boxes near 
%    boundary where we use point evaluation. Guarantees Knotenexaktheit for
%    s(x)=x on uniform grids.
%
% S = 1./[1 boxSize 1] .*[sEvaluatedAtNodes(1),  sIntegrated,  sEvaluatedAtNodes(end)];

%%%%% 
% 4) S = integral mean of s on Voronoibox with s_0 = s_{N+1} = 0
%
% S = 1./[1 boxSize 1] .*[0,  sIntegrated,  0];



%% Set up right hand side (boundary data only enters here)
%if numberInteriorGridPoints == 1 % only one interior point
    %b = cLeft* (D/gridSize(1)   * Bernoulli(-P(1))) + cRight*(D/gridSize(end) * Bernoulli( P(end)))...
    %    + gridSize(1:end-1).*V(-P(1:end-1)).*S(1:end-2) + (boxSize - gridSize(1:end-1).*V(P(1:end-1)) - gridSize(2:end).*V(-P(2:end))).*S(2:end-1) + gridSize(2:end).*V(P(2:end)).*S(3:end);
    %bhom   = boxSize.*S(2:end-1);
    %bInhom = gridSize(1:end-1).*V(-P(1:end-1)).*S(1:end-2) + (        - gridSize(1:end-1).*V(P(1:end-1)) - gridSize(2:end).*V(-P(2:end))).*S(2:end-1) + gridSize(2:end).*V(P(2:end)).*S(3:end);
%else                             % several interior points
    %b = zeros(numberInteriorGridPoints,1);
    %b = sIntegrated + gridSize(1:end-1).*V(-P(1:end-1)).*S(1:end-2) - (gridSize(1:end-1).*V(P(1:end-1)) + gridSize(2:end).*V(-P(2:end))).*S(2:end-1) + gridSize(2:end).*V(P(2:end)).*S(3:end);
    %b = gridSize(1:end-1).*V(-P(1:end-1)).*S_ev(1:end-2) + (boxSize - gridSize(1:end-1).*V(P(1:end-1)) - gridSize(2:end).*V(-P(2:end))).*S_ev(2:end-1) + gridSize(2:end).*V(P(2:end)).*S_ev(3:end);
    binterior = gridSize(1:end-1).*V(-Pe(1:end-1)).*S_ev(1:end-2) ... %changed in comparison to original version
        + (boxSize(2:end-1) - gridSize(1:end-1).*V(Pe(1:end-1)) - gridSize(2:end).*V(-Pe(2:end))).*S_ev(2:end-1) ...  
        + gridSize(2:end).*V(Pe(2:end)).*S_ev(3:end);

    b = [boxSize(1)*S_ev(1)       + gridSize(1)     * ( V(Pe(1))  *  S_ev(2) - V(-Pe(1))  *S_ev(1)     ),...
         binterior,...
         boxSize(end)*S_ev(end)   - gridSize(end)   * ( V(Pe(end))*S_ev(end) - V(-Pe(end))*S_ev(end-1) )];
    % Add two values to first and last entry of rhs since linear system is
    % reduced to (numberInteriorGridPoints x numberInteriorGridPoints) instead
    % of (numberGridPoints x numberGridPoints)
    %b(1)   = b(1)   + cLeft *(D/gridSize(1)   * Bernoulli(-P(1)));
    %b(end) = b(end) + cRight*(D/gridSize(end) * Bernoulli( P(end)));

    % % Set up rhs as for Scharfetter Gummel (for comparision)
    % bSG = sIntegrated(2:end-1);                               % bSG =(s_1,...,s_N)
    % bSG(1)   = bSG(1)   + cLeft *(D/gridSize(1)   * Bernoulli(-v/D*gridSize(1)));
    % bSG(end) = bSG(end) + cRight*(D/gridSize(end) * Bernoulli( v/D*gridSize(end)));
    % 
    % fprintf('The absolute difference for the right-hand sides between CFS and SG is: %e\n', max(abs(bSG-b)))
%end



%% Solve system
cCFS=ACFS\b';       % note b is a row vector and needs to be transposed


%% Print condition number and residual

% Condition number
if issparse(ACFS)
    cn = condest(ACFS);
else
    cn = cond(ACFS);
end

% Residual
residual = 1/size(b',1)*sum(abs(ACFS*cCFS - b'));

% 
% fprintf('The condition number of A is:     %e\n', cn)
% fprintf('The residual is:                  %e\n', residual)