function [ eta ] = Viscosity_CO2( T, P )
% Calculation of the viscosity of CO2 [Pa*s] depending on pressure and temperature
% equation: correlation of Fenghour et al. 1998
% input: temperature [K], pressure [Pa]
% output: viscosity [Pa*s]

% viscosity eta0 at 1 atm, range of validity: 200 K < T < 1500 K (Fenghour et al 1998)
 a0 = 0.235156;
 a1 = -0.491266;
 a2 = 5.211155e-2;
 a3 = 5.347906e-2;
 a4 = -1.537102e-2;
 reduced_T = T/251.196; 
 reduced_cross_section = exp (a0*log(reduced_T)^0 + a1*log(reduced_T)^1 + a2*log(reduced_T)^2 + a3*log(reduced_T)^3 + a4*log(reduced_T)^4); 
 eta0 = 1.00697*T^(1/2)/(reduced_cross_section)*10^(-6); % in Pa*s

rho = 44.01 *  Density_scCO2( T, P ); % g/m3 related to CO2
rho1 = rho/1000; % density CO2 = f (P,T) in kg/m^3

% calculation of eta1_0  (Fenghour et al 1998)
    % calculation of delta_eta1
    d11 = 0.4071119e-2;
    d21 = 0.7198037e-4;
    d64 = 0.2411697e-16;
    d81 = 0.2971072e-22;
    d82 = -0.1627888e-22;
    delta_eta1 = (d11*rho1 + d21*rho1^2 + d64*rho1^6/reduced_T^3 + d81*rho1^8 + d82*rho1^8/reduced_T)*10^-6; % in Pa*s

   % calculation of delta_eta1_c according to Vesovic et al 1990 /Luttmer-Strahtmann et al 1995
if (300 < T) && (T < 310) && (200 < rho1) && (rho1 < 600)   
eta_line = eta0 + delta_eta1; % in Pa*s
z = 0.06; s
nu = 0.630;
gamma = 1.2415;
ceta_0 = 1.5*10^-10;
Gamma = 0.052;
T_c = 304.107; 
P_c = 7.3721*10^6;
rho_c = 467.69;
T_r = 1.5*T_c;

[c_v, c_p]= Heat_capacity_CO2 (T,rho1); % calculation of the specific heat capacity according to Span and Wagner 1996
c_v = c_v;  % in J/(kg*K)
c_p = c_p;  % in J/(kg*K)

P_gradient = P - 10^5;
rho1_gradient = 44.01/1000*Density_scCO2(T,P_gradient);
gradientDrho1DP = (rho1-rho1_gradient)/(P-P_gradient);
rho1_T_r = 44.01/1000*Density_scCO2(T_r,P) ;
rho1_gradient_T_r = 44.01/1000*Density_scCO2(T_r,P_gradient);
gradientDrho1DP_T_r = (rho1_T_r-rho1_gradient_T_r)/(P-P_gradient);
X_line = (P_c/(rho_c^2*T_c))*rho1*T*gradientDrho1DP;
X_line_T_r = (P_c/(rho_c^2*T_c))*rho1_T_r*T_r*gradientDrho1DP_T_r;
delta_X_line = X_line - X_line_T_r*T_r/T;
ceta = ceta_0*(delta_X_line/Gamma)^(nu/gamma);
q_D = 1/(2.3*10^-10);

b0 = 0.4226159;
b1 = 0.6280115;
b2 = -0.5387661;
b3 = 0.6735941;
b4 = 0;
b5 = 0;
b6 = -0.4362677;
b7 = 0.2255388;

c1 = 2.387869*10^-2;
c2 = 4.350794;
c3 = -10.33404;
c4 = 7.981590;
c5 = -1.940558;

k_B = 1.3806485279*10^-23; % Boltzmann constant in J/K
c_int = k_B*(1+exp(-183.5/T)*(c1*(T/100)^(1) + c2*(T/100)^(0) + c3*(T/100)^(-1) + c4*(T/100)^(-2) + c5*(T/100)^(-3)));
r= (2*c_int/(5*k_B))^(1/2);
cross_section_lambda = b0/(reduced_T^0) + b1/(reduced_T^1) + b2/(reduced_T^2) + b3/(reduced_T^3) + b4/(reduced_T^4) + b5/(reduced_T^5) +  b6/(reduced_T^6) + b7/(reduced_T^7); 
lambda_0 = (0.475598*T^(1/2)*(1+r^2)/cross_section_lambda)*10^-3; % in W/(m*K)

d1 = 2.447164*10^-2;
d2 = 8.705605*10^-5;
d3 = -6.547950*10^-8;
d4 = 6.594919*10^-11;

delta_lambda = (d1*rho1^1 + d2*rho1^2 + d3*rho1^3 + d4*rho1^4)*10^-3; % in W/(m*K)
lambda_line = lambda_0 + delta_lambda;  % in W/(m*K)
y_D = atan(q_D*ceta);
y_delta =(atan(q_D*ceta/(1+q_D^2*ceta^2)^(1/2))-y_D)/(1+q_D^2*ceta^2)^(1/2); 
y_alpha = rho1*k_B*T/(8*pi*eta_line^2*ceta);
y_beta = lambda_line /(eta_line*(c_p - c_v));
y_gamma = c_v/(c_p - c_v);
y_eta = (y_delta + y_beta/y_alpha)/y_D;
y_nu = y_gamma*y_delta/y_D;

h = (3*y_gamma*y_eta + 3*y_eta/2 - y_eta^3 - y_nu)*y_D + (y_eta^2 - 2*y_gamma - 5/4)*sin(y_D) - y_eta*sin(2*y_D)/4 + (sin(3*y_D))/12 + (y_gamma *(1+y_gamma))^(3/2)/(y_nu - y_gamma*y_eta)*atan((y_gamma/(1+y_gamma))^(1/2)*tan(y_D));

% according to Luettmer-Strathmann et al. 1995
%z^3 + y_eta*z^2 + y_gamma*z + y_nu = 0; 
p = [1 y_eta y_gamma y_nu]; 
z_all = roots(p);
z1 = z_all(1);
z2 = z_all(2);
z3 = z_all(3);

F_z1= 1/(1-z1^2)^(1/2)*log((1-z1+(1-z1^2)^(1/2)*tan(y_D/2))/(1-z1-(1-z1^2)^(1/2)*tan(y_D/2))); 
F_z2= 1/(1-z2^2)^(1/2)*log((1-z2+(1-z2^2)^(1/2)*tan(y_D/2))/(1-z2-(1-z2^2)^(1/2)*tan(y_D/2)));
F_z3= 1/(1-z3^2)^(1/2)*log((1-z3+(1-z3^2)^(1/2)*tan(y_D/2))/(1-z3-(1-z3^2)^(1/2)*tan(y_D/2)));
C2 = y_eta^4 - 4*y_eta^2*y_gamma - 2*y_eta^2 + 2*y_eta*y_nu + 3*y_gamma^2 + 4*y_gamma + 1;
C1 = (y_eta*y_gamma - y_nu)*(y_eta^2 - 3*y_gamma -2) + y_gamma^2*(1+y_gamma)^2/(y_eta*y_gamma - y_nu);
C0 = y_eta*y_nu*(y_eta^2 -3*y_gamma -2) + y_nu^2 + y_nu*y_gamma*(1+y_gamma)^2/(y_eta*y_gamma -y_nu);

H = h + (C2*z1^2 + C1*z1 + C0)/((z1-z2)*(z1-z3))*F_z1 + (C2*z2^2 + C1*z2 + C0)/((z2-z1)*(z2-z3))*F_z2 + (C2*z3^2 + C1*z3 + C0)/((z3-z1)*(z3-z2))*F_z3;

delta_eta1_c = eta_line*exp(z*H)-eta_line; % in Pa*s  
delta_eta1_c = real(delta_eta1_c); % delete the imaginary part (in the range of 10^-22)
else
    delta_eta1_c = 0;
end 

    % calculation of eta1
    eta = eta0 + delta_eta1 + delta_eta1_c; % CO2 viscosity in Pa*s

end

