function [x_EtOH] = cal_x_EtOH_from_rho_EtOH_PR_MR2( T, P, rho_EtOH)
% Calculation of the molar fraction of ethanol from density of EtOH (PR EOS with Mixture Rule 2)
% 1:CO2, 2:ETOH  i:CO2, j:ETOH
% rho_EtOH in mol/m^3

% Fitting parameter delta_ij, delta_ji for PR EOS using mixture rule 2 (Selmer et al. 2018, Supplementary information)
delta_ij = 0.156;
delta_ji = 0.758;

if rho_EtOH <= 1e-6
    x_EtOH = 0; 
else
    R = 8.3144621;   

    c_EtOH = -4.275e-06; % fitted parameter c for pure EtOH
    k = P/(R*T);
    g = (1/rho_EtOH - c_EtOH) *k;
    h = P/(R*T)^2;

    % CO2 properties:
    Tc1 =  304.12; % K   
    Pc1 = 7.376e6 ; % Pa  
    % reduced values
    Pr1 = P/Pc1;
    Tr1 = T/Tc1;
    % fitted parameters
    B1 = 1.0010;
    B2 = 0.0524;
    B3 = 0.2315;
    B4 = 0.1286;
    B5 = 0.3258;
    B6 = 0.0704;
    B7 = 0.9461;

    alpha1 = B1 + log(Pr1) * (B2 + B3* log(Tr1) -B4 * (log(Tr1))^2 - B5 * sin(B6 * (log(Pr1))^2)) - B7 * sin(log(Tr1));
    a(1) = 0.45724 * R^2 * Tc1^2 * alpha1 / Pc1;
    b(1) = 0.07780 * R * Tc1 / Pc1;
    
    % Ethanol properties:
    Tc2 = 513.92; % K
    Pc2 = 6.148e6; % Pa
    omega2 = 0.649; 

    kappa2 = 0.37464 + 1.54226*omega2 - 0.26992*omega2^2;
    alpha2 = (1 + kappa2 * ( 1 - (T/Tc2).^0.5 )).^2;
    a(2) = 0.45724 * R^2 * Tc2^2 * alpha2 / Pc2;
    b(2) = 0.07780 * R * Tc2 / Pc2;

    aiaj = sqrt(a(1) * a(2));
    a_i = a(1);
    a_j = a(2);
    b_i = b(1);
    b_j = b(2);
    diff_b = b_j-b_i; 

    diff_delta = delta_ij-delta_ji;
    diff_delta2 = 2*delta_ij-delta_ji-1;

    cubic_equ_a4 = -2*g*h*aiaj*diff_delta + h*k*2*aiaj*diff_b*diff_delta;   
    cubic_equ_a3 = g^3 + g^2*k*diff_b + g*h*(a_i+a_j) + 2*g*h*aiaj*diff_delta2 - g*k^2*3*diff_b^2 + h*k*2*b_i*aiaj*diff_delta - h*k*a_i*diff_b - h*k*a_j*diff_b - h*k*2*diff_b*aiaj*diff_delta2 + k^3*diff_b^3;   
    cubic_equ_a2 = g^2*(k*b_i-1) - 2*g*h*a_i + 2*g*h*aiaj*(1-delta_ij) - g*k^2*6*b_i*diff_b - g*k*2*diff_b - h*k*(b_i*a_i + b_i*a_j) - h*k*2*b_i*aiaj*diff_delta2 + h*k*2*diff_b*a_i - h*k*2*diff_b*aiaj*(1-delta_ij) + k^2*diff_b^2 + 3*k^3*b_i*diff_b^2;
    cubic_equ_a1 = a_i*g*h - g*k^2*3*b_i^2 - g*k*2*b_i - h*k*a_i*diff_b + h*k*2*b_i*a_i - h*k*2*b_i*aiaj*(1-delta_ij) + k^2*2*b_i*diff_b + k^3*3*b_i^2*diff_b;
    cubic_equ_a0 = -h*k*a_i*b_i + k^3*b_i^3 + k^2*b_i^2;

    x_EtOH_all = roots([cubic_equ_a4 cubic_equ_a3 cubic_equ_a2 cubic_equ_a1 cubic_equ_a0]);  
    for j=1:size(x_EtOH_all,1) 
        if isreal(x_EtOH_all(j)) && (0 <= x_EtOH_all(j)) && (x_EtOH_all(j) <= (1 + 1e-1))
            x_EtOH = x_EtOH_all(j);
        end
    end

    if x_EtOH > 1
        x_EtOH = 1;
    end
end