function [x1, y1] = flash_PR_MR2(input)
% Calculation of the composition of gas and liquid phase in two phase region using a flash_PR_MR2
% for the system ethanol - CO2 using PR EOS with vDW mixing rules
% mixing rule 2
% input: input vector: temperature [K], pressure [Pa], total composition z [mol/mol] 
% output: composition of the liquid phase x [mol/mol], composition of the gas phase y [mol/mol]

% input 
T = input(1);  % K
P = input(2);  % Pa

z(1) = input(3);
                k12 = 0.0488;
                k21 = 0.0990;
                l12 = 0.0527;
                l21 = 0.0527;
z(2) = 1 - z(1);

%% global definitions

R = 8.3144621;         % [J mol^-1 K^-1]
    
%----------------------------------------------------------------------------------------
% DEFINE INPUT
%----------------------------------------------------------------------------------------

% number of species in the mixture
no_species = 2;

% define the component properties
% CO2:
Tc(1) =  304.2; % K
Pc(1) = 73.76e5 ; % Pa
w(1) = 0.225;
% source: NIST temperature range: 154.26 - 195.89 K
A_antoine(1)=6.81228;
B_antoine(1)=1301.679;
C_antoine(1)=-3.494;

% Ethanol
Tc(2) = 513.92; % K
Pc(2) = 61.4e5; %Pa
w(2) = 0.644;
% source: NIST temperature range: 300.9 - 453.5 K
A_antoine(2)= 5.246777;
B_antoine(2)= 1598.763;
C_antoine(2)= -46.424;

% binary interaction parameter
k(1,2) = k12;
k(2,1) = k21;
l(1,2) = l12;
l(2,1) = l21;

% calculate PR-EOS coefficients of pure components
for i=1:(no_species)
    kappa(i) = 0.37464 + 1.54226*w(i) - 0.26992*w(i)^2;
    alpha(i) = (1. + kappa(i) * ( 1. - sqrt(T/Tc(i)) ))^2;

    % PR EOS
    a(i) = 0.45724 * (R * Tc(i))^2 * alpha(i) / Pc(i);
    b(i) = 0.07780 * R * Tc(i) / Pc(i);   
end


% inital guess K_partition_coef with Antoine (assuming ideal phases)

for i=1:no_species
    p_vap_antoine = 10 ^ (A_antoine(i) - B_antoine(i) / (T + C_antoine(i)) );
    K_partition_coef(i) = p_vap_antoine/(P*1e-5);
end


 % guess beta
beta_min = 1/(1 - max(K_partition_coef));
beta_max = 1/(1 - min(K_partition_coef));
beta = 0.5*(beta_min + beta_max);

stop_criterion = 0.0001;

%----------------------------------------------------------------------------------------
% START CALCULATION
%----------------------------------------------------------------------------------------

% iteration
count_iterations = 0;
target_function = stop_criterion + 1;
while (target_function > stop_criterion)
    count_iterations = count_iterations + 1;

    % calculate new beta with Rachford-Rice equation (inner iteration)

    count_beta_iter = 0;
    target_beta_iterartion = stop_criterion + 1;
    while (target_beta_iterartion > stop_criterion)
        count_beta_iter = count_beta_iter + 1;
        rr = 0;
        derivative_rr = 0;
        for i=1:no_species
            rr = rr + (z(i) * (K_partition_coef(i) - 1)) / (1 + beta*(K_partition_coef(i) - 1) );
            derivative_rr = derivative_rr + (-z(i) * (K_partition_coef(i) - 1)^2) / (1 + beta*(K_partition_coef(i) - 1) )^2;
        end
        % new beta with Newton-Rapshon
        beta_new = beta - rr / derivative_rr;
        target_beta_iterartion = abs( (beta_new-beta)/beta );
        beta = beta_new;
    end
                  

    % calculate compositions

    for  i=1:no_species
        x(i) = z(i) / (1 + beta * (K_partition_coef(i) - 1) );
        y(i) = x(i) * K_partition_coef(i);
    end



    %------------------------------------------------------------------------------------
    % *** VAPOR PHASE ***
    %------------------------------------------------------------------------------------

    % use EOS mixing rules to calculate parameters


    a_mix = 0;
    b_mix = 0;
    for i=1:no_species
        for j=1:no_species
            a_ij = sqrt(a(i) * a(j)) * (1 - y(i)*k(i,j)-y(j)*k(j,i));         
            a_mix = a_mix + y(i) * y(j) * a_ij;
            b_ij = ((b(i) + b(j))/2)*(1 -l(i,j));
            b_mix = b_mix + x(i) * x(j) * b_ij;
        end
    end

    
    % PR EOS
    A_mix = a_mix * P / (R * T)^2; 
    B_mix = b_mix * P / (R * T);

    % PR EOS           
    cubic_equ_a3 = 1;   
    cubic_equ_a2 = B_mix - 1;
    cubic_equ_a1 = A_mix - 3*B_mix^2 - 2*B_mix;
    cubic_equ_a0 = -A_mix*B_mix + B_mix^2 + B_mix^3;

    % solve the cubic equation of state: a3*Z**3 + a2*Z**2 + a1*Z + a0 = 0
    % the input list for the function real_roots needs a list of the coefficients a3 - a0
    list_of_Z = roots([cubic_equ_a3 cubic_equ_a2 cubic_equ_a1 cubic_equ_a0]);  
    all_results = list_of_Z; 
    real_results = 0;
    for i=1:size(all_results,1)             
        if isreal(all_results(i))
            real_results = [real_results; all_results(i)];
        end
    end
    real_results(1) = [];
    list_of_Z = real_results;

    Z_vapor  = max(list_of_Z);
    
    for i=1:no_species
        sum_ya = 0;
        for j=1:no_species
            a_ij = sqrt(a(i)*a(j)) * (1 - y(i)*k(i,j)-y(j)*k(j,i));
            sum_ya = sum_ya + y(j) * a_ij;
        end      

        % mixture fugacity coefficient calculated with the PR EOS
        fcoeff_vapor(i) = exp( (b(i)/b_mix) * (Z_vapor - 1) - log(Z_vapor - B_mix) - A_mix/(2*sqrt(2)*B_mix) * (2*sum_ya/a_mix - b(i)/b_mix) * log( (Z_vapor + (1+sqrt(2))*B_mix) / (Z_vapor + (1-sqrt(2))*B_mix) )  );
        fugacity_vapor(i) = fcoeff_vapor(i) * P * y(i);
    end 

    



    %------------------------------------------------------------------------------------
    % *** LIQUID PHASE ***
    %------------------------------------------------------------------------------------

    % use EOS mixing rules to calculate parameters            
    a_mix = 0;
    b_mix = 0;
    for i=1:no_species
        for j=1:no_species
            a_ij = sqrt(a(i) * a(j)) * (1 - x(i)*k(i,j)-x(j)*k(j,i));
            a_mix = a_mix + x(i) * x(j) * a_ij;
            b_ij = ((b(i) + b(j))/2)*(1 -l(i,j));
            b_mix = b_mix + x(i) * x(j) * b_ij;
        end
    end
    
    % PR EOS
    A_mix = a_mix * P / (R * T)^2; 
    B_mix = b_mix * P / (R * T);

    % PR EOS            
    cubic_equ_a3 = 1;   
    cubic_equ_a2 = B_mix - 1;
    cubic_equ_a1 = A_mix - 3*B_mix.^2 - 2*B_mix;
    cubic_equ_a0 = -A_mix*B_mix + B_mix^2 + B_mix^3;

    
    % solve the cubic equation of state: a3*Z**3 + a2*Z**2 + a1*Z + a0 = 0
    % the input list for the function real_roots needs a list of the coefficients a3 - a0
    list_of_Z = roots([cubic_equ_a3 cubic_equ_a2 cubic_equ_a1 cubic_equ_a0]); 
    all_results = list_of_Z; 
    real_results = 0;
    for i=1:size(all_results,1)             
        if isreal(all_results(i))
            real_results = [real_results; all_results(i)];
        end
    end
    real_results(1) = [];
    list_of_Z = real_results;

    Z_liquid  = min(list_of_Z);

    for i=1:no_species
        sum_ya = 0;
        for j=1:no_species
            a_ij = sqrt(a(i)*a(j)) * (1 - x(i)*k(i,j)-x(j)*k(j,i));
            sum_ya = sum_ya + x(j) * a_ij;
        end
        % mixture fugacity coefficient calculated with the PR EOS
        fcoeff_liquid(i) = exp((b(i)/b_mix) * (Z_liquid - 1) - log(Z_liquid - B_mix) - A_mix/(2*sqrt(2)*B_mix) * (2*sum_ya/a_mix - b(i)/b_mix) *  log( (Z_liquid + (1+sqrt(2))*B_mix) / (Z_liquid + (1-sqrt(2))*B_mix) )  );
        fugacity_liquid(i) = fcoeff_liquid(i) * P * x(i);
    end 

    %------------------------------------------------------------------------------------
    % *** calculate new partition coefficients (phase equilibrium) ***
    %------------------------------------------------------------------------------------

    % new partition coef.
    for i=1:no_species
        K_partition_coef(i) = fcoeff_liquid(i) / fcoeff_vapor(i);
    end

    % calculate target function
    target_function = 0;
    for i=1:no_species
        target_function = target_function + x(i) * K_partition_coef(i);
    end
    target_function = abs(target_function - 1);

    i = 1;
    str1 = ['iteration loop:' num2str(count_iterations) ' target =' num2str(target_function) ' beta =' num2str(beta) ' Vapor: y =' num2str(y(i)) ' Z =' num2str(Z_vapor) ' f =' num2str(fugacity_vapor(i)*10^-5) ' Liquid: x =' num2str(x(i)) ' Z =' num2str(Z_liquid) ' f =' num2str(fugacity_liquid(i)*10^-5)];


end
x1 = x(1);
y1 = y(1);


